# Generated by Django 2.2.20 on 2021-04-07 20:04

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0126_auto_20210308_1522'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecatalogquery',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4, null=True),
        ),
    ]
